<?php

namespace Geral\Logic\PropostaComercial;

use Faturamento\Entities\Contratolinp;
use Faturamento\Entities\Valorlinp;
use Faturamento\Entities\Valorserv;
use Geral\Entities\Contratoserv;
use Geral\Entities\Procomerci;
use Geral\Entities\Prospect;
use Geral\Util\Ambiente;
use Geral\Util\DateUtil;
use Geral\Entities\Cliente;
use Geral\Entities\Endeclie;
use Geral\Entities\Txtpropfas;
use Geral\Entities\Opergrupop;
use Geral\Entities\Propostafas;
use Academico\Entities\Endeprospect;
use Faturamento\Entities\Obspropfas;
use Faturamento\Entities\Propostalinp;
use Faturamento\Entities\Propostaserv;
use Faturamento\Entities\Sistproplinp;
use Faturamento\Entities\Venclusoprop;
use Indeterminado\Entities\Objetpropfas;
use Indeterminado\Entities\Vencservprop;
use Producao\Entities\Autgrupop;
use Setforms2\Shared\Registry;
use Setforms2\Runtime\Helper\Procedure;
use Setforms2\Runtime\Helper\Procedure\Param;
use Geral\Logic\PropostaComercial\Exception\RestricoesPropostaComercial;

class PropostaComercial {
    public static function checkCadastroProposta($nrProposta, $nrVersao, $nrOrg) {
        $proposta = Propostafas::findOneByNrproposfasAndNrverpropfasAndNrorg($nrProposta, $nrVersao, $nrOrg);

        if ($proposta){
            return true;
        }
        return false;
    }

    public static function getIdiomaClienteProspect($tipo, $codigo) {
        if ($tipo == 'C') {
            /** @var $cliente \Geral\Entities\Cliente */
            $cliente = Cliente::findOneByCdcliente($codigo);

            /** @var $endereco \Geral\Entities\Endeclie */
            $endereco = Endeclie::findOneByCdclienteAndIdtpendeclie($codigo, 'P');

            $pais = $endereco->getCdpais();
            $idioma = $cliente->getIdidioma();

            return self::idiomaClienteProspect($pais, $idioma);
        } else {
            /** @var $prospect \Geral\Entities\Prospect */
            $prospect = \Geral\Entities\Prospect::findOneByCdprospect($codigo);

            /** @var $endereco \Academico\Entities\Endeprospect */
            $endereco = Endeprospect::findOneByCdprospectAndIdtpendepros($codigo, 'P');

            $pais = $endereco->getCdpais();
            $idioma = $prospect->getIdidioma();

            return self::idiomaClienteProspect($pais, $idioma);
        }
    }

    public static function montaCamposNovaProposta($campos) {
        if ($campos['IDCLIENPROSP'] == 'P') {
            /** @var $prospect \Geral\Entities\Prospect */
            $prospect = \Geral\Entities\Prospect::findOneByCdprospect($campos['CDCLIPROSP']);

            /** @var $endereco \Academico\Entities\Endeprospect */
            $endereco = \Academico\Entities\Endeprospect::findOneByCdprospectAndIdtpendepros($campos['CDCLIPROSP'], 'P');

            $campos['CDPROSPECT'] = $campos['CDCLIPROSP'];
            $campos['CDCLIENTE'] = '';
            $campos['NRMESREAJPROP'] = '';

            $pais = $endereco->getCdpais();
            $idioma = $prospect->getIdidioma();

            $campos['IDIDIOMA'] = self::idiomaClienteProspect($pais, $idioma);
        } else {
            /** @var $cliente \Geral\Entities\Cliente */
            $cliente = Cliente::findOneByCdcliente($campos['CDCLIPROSP']);

            /** @var $endereco \Geral\Entities\Endeclie */
            $endereco = Endeclie::findOneByCdclienteAndIdtpendeclie($campos['CDCLIPROSP'], 'P');

            $campos['CDCLIENTE'] = $campos['CDCLIPROSP'];
            $campos['CDPROSPECT'] = '';
            $campos['NRMESREAJPROP'] = $cliente->getNrmesreajclie();

            $pais = $endereco->getCdpais();
            $idioma = $cliente->getIdidioma();

            $campos['IDIDIOMA'] = self::idiomaClienteProspect($pais, $idioma);
        }

        $indice = Cliente::getRepository()->indicesReajuste($campos['CDCLIENTE']);

        $dataAtual = DateUtil::getDataAtual();

        $campos['DTINCLUSAO']   = $dataAtual;
        $campos['DSOBSPROPFAS'] = 'SITE';
        $campos['IDSOLPROPFAS'] = 'S';
        $campos['IDTPREAJPROP'] = 'A';
        $campos['CDTPINDICE1']  = $indice[0]['CDTPINDICE1PROP'];
        $campos['IDPERVALOR1']  = $indice[0]['IDPERVALOR1PROP'];
        $campos['CDINDICE1']    = $indice[0]['CDINDICE1PROP'];
        $campos['VRPEINCIND1']  = $indice[0]['VRPEINCIND1PROP'];
        $campos['CDTPINDICE2']  = $indice[0]['CDTPINDICE2PROP'];
        $campos['IDPERVALOR2']  = $indice[0]['IDPERVALOR2PROP'];
        $campos['CDINDICE2']    = $indice[0]['CDINDICE2PROP'];
        $campos['VRPEINCIND2']  = $indice[0]['VRPEINCIND2PROP'];

        return $campos;
    }

    public static function getCamposNovaProposta($tipo, $cdcliente) {
        if ($tipo == 'C') {
            $indice = self::indicesReajuste($cdcliente);

            $campos['CDTPINDICE1'] = $indice[0]['CDTPINDICE1PROP'];
            $campos['IDPERVALOR1'] = $indice[0]['IDPERVALOR1PROP'];
            $campos['CDINDICE1']   = $indice[0]['CDINDICE1PROP'];
            $campos['VRPEINCIND1'] = $indice[0]['VRPEINCIND1PROP'];
            $campos['CDTPINDICE2'] = $indice[0]['CDTPINDICE2PROP'];
            $campos['IDPERVALOR2'] = $indice[0]['IDPERVALOR2PROP'];
            $campos['CDINDICE2']   = $indice[0]['CDINDICE2PROP'];
            $campos['VRPEINCIND2'] = $indice[0]['VRPEINCIND2PROP'];

            /** @var $cliente \Geral\Entities\Cliente */
            $cliente = \Geral\Entities\Cliente::findOneByCdcliente($cdcliente);

            $campos['MESREAJUSTE'] = $cliente->getNrmesreajclie();

            return $campos;
        } else {
            $indice = self::indicesReajuste('');

            $campos['CDTPINDICE1'] = $indice[0]['CDTPINDICE1PROP'];
            $campos['IDPERVALOR1'] = $indice[0]['IDPERVALOR1PROP'];
            $campos['CDINDICE1']   = $indice[0]['CDINDICE1PROP'];
            $campos['VRPEINCIND1'] = $indice[0]['VRPEINCIND1PROP'];
            $campos['CDTPINDICE2'] = $indice[0]['CDTPINDICE2PROP'];
            $campos['IDPERVALOR2'] = $indice[0]['IDPERVALOR2PROP'];
            $campos['CDINDICE2']   = $indice[0]['CDINDICE2PROP'];
            $campos['VRPEINCIND2'] = $indice[0]['VRPEINCIND2PROP'];

            $campos['MESREAJUSTE'] = '';

            return $campos;
        }
    }

    public static function indicesReajuste($cdcliente) {
        return Propostafas::getRepository()->indicesReajuste($cdcliente);
    }

    public static function idiomaClienteProspect($pais, $idioma) {
        if (!empty($pais) && $pais == '0055')
            return 'POR';
        elseif (!empty($idioma))
            return $idioma;

        return 'ESP';
    }

    public static function novaPropostaComercial($campos, $notDuplicaTexto = null) {
        self::inserePropostaComercial($campos);
        self::insereObservacao($campos);
        self::insereObjetivo($campos);

        if(is_null($notDuplicaTexto)) {
            self::insereTextoPadraoPropostaComercial($campos);
        }
    }

    public static function inserePropostaComercial($campos) {
        $manager = \Setforms2\Shared\Registry::getEntityManager();

        $proposta = new Propostafas();
        $proposta->setNrorg($campos['NRORG']);
        $proposta->setNrproposfas($campos['NRPROPOSFAS']);
        $proposta->setNrverpropfas($campos['NRVERPROPFAS']);
        $proposta->setIdclienprosp($campos['IDCLIENPROSP']);
        $proposta->setDtproposfas($campos['DTPROPOSFAS']);
        $proposta->setDtvalpropfas($campos['DTVALPROPFAS']);
        $proposta->setDsemaiprofas($campos['DSEMAIPROFAS']);
        $proposta->setNrfaxpropfas($campos['NRFAXPROPFAS']);
        $proposta->setCdoperincl($campos['CDOPERINCL']);
        $proposta->setCdopersoli($campos['CDOPERSOLI']);
        $proposta->setCdoperresp($campos['CDOPERSOLI']);
        $proposta->setCdfilial($campos['CDFILIAL']);
        $proposta->setCdprospect($campos['CDPROSPECT']);
        $proposta->setCdintepros($campos['CDINTEPROS']);
        $proposta->setCdcliente($campos['CDCLIENTE']);
        $proposta->setCdinteclie($campos['CDINTECLIE']);
        $proposta->setNrmesreajprop($campos['NRMESREAJPROP']);
        $proposta->setDsobspropfas($campos['DSOBSPROPFAS']);
        $proposta->setIdsolpropfas($campos['IDSOLPROPFAS']);
        $proposta->setIdtpreajprop($campos['IDTPREAJPROP']);
        $proposta->setCdoperresp($campos['CDOPERRESP']);
        $proposta->setIdtipopropfas($campos['IDTIPOPROPFAS']);
        $proposta->setDsrefeproposfas($campos['DSREFEPROPOSFAS']);
        $proposta->setDtinclusao($campos['DTINCLUSAO']);
        $proposta->setCdprocomerci($campos['CDPROCOMERCI']);
        $proposta->setNrcontato($campos['NRCONTATO']);
        $proposta->setIdtipocobrserv($campos['IDTIPOCOBRSERV']);
        $proposta->setCdclienterevenda($campos['CDCLIENTEREVENDA']);
        $proposta->setCdinteclierevenda($campos['CDINTECLIEREVENDA']);
        $proposta->setCdtpindice1($campos['CDTPINDICE1']);
        $proposta->setIdpervalor1($campos['IDPERVALOR1']);
        $proposta->setCdindice1($campos['CDINDICE1']);
        $proposta->setVrpeincind1($campos['VRPEINCIND1']);
        $proposta->setCdtpindice2($campos['CDTPINDICE2']);
        $proposta->setIdpervalor2($campos['IDPERVALOR2']);
        $proposta->setCdindice2($campos['CDINDICE2']);
        $proposta->setVrpeincind2($campos['VRPEINCIND2']);
        $proposta->setIdliberaagendamento($campos['IDLIBERAAGENDAMENTO']);

        $proposta->persist();
        $manager->flush();

    }

    public static function insereObservacao($campos) {
        Obspropfas::getRepository()->insertObservacaoPropostaComercial($campos);
    }

    public static function insereObjetivo($campos) {
        if ($campos['IDTIPOPROPFAS'] == 'S' && $campos['TXOBJETPROPFAS'] != '') {
            Objetpropfas::getRepository()->insertObjetivoPropostaComercial($campos);
        }
    }

    public static function insereTextoPadraoPropostaComercial($campos) {
        $where = "CDPROCOMERCI = ?
                  AND IDIDIOMA = ?
                  AND IDTIPPROPROCOME = ?";

        $param = array(
            $campos['CDPROCOMERCI'],
            $campos['IDIDIOMA'],
            $campos['IDTIPOPROPFAS']
        );

        $buscaTextoPadrao = \Geral\Entities\Txtproprodcomer::getObjectsByWhere($where, $param);

        $entityManager = Registry::getEntityManager();
        $db = $entityManager->getConnection();
        $db->beginTransaction();
        
        try {
            foreach ($buscaTextoPadrao as $texto) {
                $textoPadrao = new \Geral\Entities\Txtpropfas();
                $textoPadrao->setNrorg($campos['NRORG']);
                $textoPadrao->setNrproposfas($campos['NRPROPOSFAS']);
                $textoPadrao->setNrverpropfas($campos['NRVERPROPFAS']);
                $textoPadrao->setCdtxtpropfas($texto->getCdtxtproprocome());
                $textoPadrao->setNrordemmsg($texto->getNrordtxtprocome());
                $textoPadrao->setTxpropfasClob($texto->getTxproprocome());
                $textoPadrao->setTxpropfas('Texto');
                $textoPadrao->setIdtipotxtpropfas('EDICAO');
                $textoPadrao->setDtinclusao($campos['DTINCLUSAO']);
                
                $textoPadrao->persist();    
            }      
            
            $entityManager->flush();
            $db->commit();
        } catch (\Exception $e) {
            $db->rollback();
            tk_warning($e->getMessage());
            throw $e;
        }
    }

    public static function setTxpropfas($nrProposta, $nrVersao, $nrOrg, $idTipo)
    {
        $entityManager = Registry::getEntityManager();
        $db = $entityManager->getConnection();
        $db->beginTransaction();
        
        $txtProposta = Txtpropfas::findByNrproposfasAndNrverpropfasAndNrorgAndIdtipotxtpropfas($nrProposta, $nrVersao, $nrOrg, $idTipo);

        try {
            foreach ($txtProposta as $txt) {
                $dados = array("NRPROPOSFAS"      => $nrProposta,
                               "NRVERPROPFAS"     => $nrVersao,
                               "CDTXTPROPFAS"     => $txt->getCdtxtpropfas(),
                               "TXPROPFAS_CLOB"   => $txt->getTxpropfasClob(),
                               "IDTIPOTXTPROPFAS" => $idTipo,
                               "NRORG"            => $nrOrg
                );

                self::procedure_SET_TXPROPFAS($dados, 'LR');
            }

            $db->commit();
        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }
    }

    public static function setTxpropfasClob($nrProposta, $nrVersao, $nrOrg, $idTipo)
    {
        $txtProposta = Txtpropfas::findByNrproposfasAndNrverpropfasAndNrorgAndIdtipotxtpropfas($nrProposta, $nrVersao, $nrOrg, $idTipo);

        try {
            foreach ($txtProposta as $txt) {
                $dados = array("NRPROPOSFAS"      => $nrProposta,
                               "NRVERPROPFAS"     => $nrVersao,
                               "CDTXTPROPFAS"     => $txt->getCdtxtpropfas(),
                               "TXPROPFAS_CLOB"   => $txt->getTxpropfasClob(),
                               "IDTIPOTXTPROPFAS" => $idTipo,
                               "NRORG"            => $nrOrg
                );

                self::procedure_SET_TXPROPFAS($dados, 'CLOB');
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /*
     * @param $typeColum -> Tipo de coluna para se definir qual procedure utilizar ('LR' = LONGRAW OU 'CLOB')
     */
    public static function procedure_SET_TXPROPFAS($dados, $typeColum)
    {
        if($typeColum == 'LR') {
            $procedure = new Procedure('SET_TXPROPFAS_LR');
        } else {
            $procedure = new Procedure('SET_TXPROPFAS_CLOB');
        }

        $procedure->addParam(
            new Param(
            'P_NRPROPOSFAS', Param::PARAM_INPUT, $dados['NRPROPOSFAS'], Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
            'P_NRVERPROPFAS', Param::PARAM_INPUT, $dados['NRVERPROPFAS'], Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
            'P_CDTXTPROPFAS', Param::PARAM_INPUT, $dados['CDTXTPROPFAS'], Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
            'P_IDTIPOTXTPROPFAS', Param::PARAM_INPUT, $dados['IDTIPOTXTPROPFAS'], Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
            'P_NRORG', Param::PARAM_INPUT, $dados['NRORG'], Param::PARAM_TYPE_INT
            )
        );
        $procedure->call();
    }

    public static function concluiProposta($proposta, $versao, $tipoProposta) 
    {
	self::validaConclusaoProposta($proposta, $versao, $tipoProposta);		

        $manager = \Setforms2\Shared\Registry::getEntityManager();

        /** @var $propostafas \Geral\Entities\Propostafas */
        $propostafas = Propostafas::findOneByNrproposfasAndNrverpropfas($proposta, $versao);

        $propostafas->setIdsolpropfas('N');

        $propostafas->persist();

        $manager->flush();
    }
    
    public static function linhaProduto($proposta, $versao) {
        $where = "NRPROPOSFAS       = ?
                  AND NRVERPROPFAS  = ?
                  AND NRORG         = 1";

        $param = array(
            $proposta,
            $versao
        );

        return Propostalinp::getObjectsByWhere($where, $param);
    }

    public static function validaConclusaoProposta($proposta, $versao, $tipoProposta) {
        $dataAtual = DateUtil::getDataAtual();
        /** @var $linhaProduto \Faturamento\Entities\Propostalinp */
        $linhaProduto = self::linhaProduto($proposta, $versao);

        //Proposta do Tipo LU não podem ser concluídas sem Linhas de Produto
        if($tipoProposta == 'M') {
            if (empty($linhaProduto)) {
                RestricoesPropostaComercial::semLinhaProduto();
            }
        }
        
        if (empty($linhaProduto)) {
            /** @var $propostaServ \Faturamento\Entities\Propostaserv */
            $propostaServ = self::propostaServ($proposta, $versao);
            if (empty($propostaServ))
                RestricoesPropostaComercial::semServicoLinhaProduto();
        }

        foreach ($linhaProduto as $linha) {
            //Phillipe Silva - 26/11/14
            //Conforme concluído junto ao Depto. de Contratos, não será mais barrada a conclusão de Propostas que
            //possuírem Linhas de Produtos com valores zerados.

            /** @var $linha \Faturamento\Entities\Propostalinp */
//            if (($linha->getVrlicencusoprop() == '0' && ($linha->getIdtiplinpp() == 'M' || $linha->getIdtiplinpp() == 'P'))
//             || ($linha->getVrmensalidaprop() == '0' && ($linha->getIdtiplinpp() == 'L' || $linha->getIdtiplinpp() == 'A')))
//                RestricoesPropostaComercial::semNegociacao();

            $parcelasLinhaProduto = Venclusoprop::getRepository()->parcelasLinhaProduto($proposta, $versao, $linha->getCdlinpprop());

            if ($parcelasLinhaProduto && DateUtil::getQtdeDiasEntre($dataAtual, DateUtil::getDataDeString(
                                    $parcelasLinhaProduto, DateUtil::FORMATO_BRASILEIRO_DATAHORA)) < 7) {
                $prazoMinimo = DateUtil::adicionaIntervalo(DateUtil::getDataAtual(), 7);
                RestricoesPropostaComercial::linhaProdutoPrazoMinimo(
                        array($parcelasLinhaProduto, $linha->getNmlinpprop(), $prazoMinimo));
            }
        }

        $parcelasServicos = Vencservprop::getRepository()->parcelasServicos($proposta, $versao);
        if ($parcelasServicos && DateUtil::getQtdeDiasEntre($dataAtual, DateUtil::getDataDeString($parcelasServicos, DateUtil::FORMATO_BRASILEIRO_DATAHORA)) < 7) {
            $prazoMinimo = DateUtil::adicionaIntervalo($dataAtual, 7);
            RestricoesPropostaComercial::servicoPrazoMinimo(array($parcelasServicos, $prazoMinimo));
        }
    }

    public static function propostaServ($proposta, $versao) {
        $where = "NRPROPOSFAS       = ?
                  AND NRVERPROPFAS  = ?";

        $param = array(
            $proposta,
            $versao
        );

        return Propostaserv::getObjectsByWhere($where, $param);
    }
    
    public static function liberaAgendamento($proposta, $versao) {
        $manager = \Setforms2\Shared\Registry::getEntityManager();

        /** @var $propostafas \Geral\Entities\Propostafas */
        $propostafas = Propostafas::findOneByNrproposfasAndNrverpropfas($proposta, $versao);

        $propostafas->setIdliberaagendamento('S');

        $propostafas->persist();

        $manager->flush();
    }    
    
    public static function getDadosProposta($nrProposta, $nrVersao, $nrOrg, $nrNovaVersao, $idClienProsp, $tipoProp, $tipoCobr) {
        $dadosPropDuplicar = Propostafas::findOneByNrproposfasAndNrverpropfasAndNrorg($nrProposta, $nrVersao, $nrOrg);
        $dadosTxtproposta  = Obspropfas::getRepository()->getTxObsProposta($nrProposta, $nrVersao, $nrOrg);
        $dadosObjproposta  = Objetpropfas::getRepository()->getTxObjetProposta($nrProposta, $nrVersao, $nrOrg);        

        if ($idClienProsp == 'C') {
            $cliProsp = $dadosPropDuplicar->getCdcliente();
        } else {
            $cliProsp = $dadosPropDuplicar->getCdprospect();
        }

        if ($dadosTxtproposta) {
            $txtprop = $dadosTxtproposta[0]['TXOBSPROPFAS'];
        } else {
            $txtprop = '';
        }

        if ($dadosObjproposta) {
            $txobj = $dadosObjproposta[0]['TXOBJETPROPFAS'];
        } else {
            $txobj = '';
        }

        $campos = array(
            'IDLIBERAAGENDAMENTO' => 'N',
            'TXOBJETPROPFAS'      => $txobj,
            'TXOBSPROPFAS'        => $txtprop,
            'IDTIPOPROPFAS'       => $tipoProp,
            'IDTIPOCOBRSERV'      => $tipoCobr,
            'CDCLIPROSP'          => $cliProsp,
            'NRPROPOSFAS'         => $nrProposta,
            'NRVERPROPFAS'        => $nrNovaVersao,
            'NRORG'               => $dadosPropDuplicar->getNrorg(),
            'CDFILIAL'            => $dadosPropDuplicar->getCdfilial(),
            'NRCONTATO'           => $dadosPropDuplicar->getNrcontato(),
            'CDOPERINCL'          => $dadosPropDuplicar->getCdoperincl(),
            'CDOPERSOLI'          => $dadosPropDuplicar->getCdopersoli(),
            'CDOPERRESP'          => $dadosPropDuplicar->getCdoperresp(),
            'CDINTECLIE'          => $dadosPropDuplicar->getCdinteclie(),
            'CDINTEPROS'          => $dadosPropDuplicar->getCdintepros(),
            'DTPROPOSFAS'         => DateUtil::getDataAtual(true),
            'DTVALPROPFAS'        => DateUtil::adicionaIntervalo(DateUtil::getDataAtual(true), 30, DateUtil::DIA),
            'IDCLIENPROSP'        => $dadosPropDuplicar->getIdclienprosp(),
            'DSEMAIPROFAS'        => $dadosPropDuplicar->getDsemaiprofas(),
            'NRFAXPROPFAS'        => $dadosPropDuplicar->getNrfaxpropfas(),
            'CDPROCOMERCI'        => $dadosPropDuplicar->getCdprocomerci(),
            'DSREFEPROPOSFAS'     => $dadosPropDuplicar->getDsrefeproposfas(),
            'CDCLIENTEREVENDA'    => $dadosPropDuplicar->getCdclienterevenda(),
            'CDINTECLIEREVENDA'   => $dadosPropDuplicar->getCdinteclierevenda()
        );

        return $campos;
    }

    public static function DuplicaLinhaProduto($nrProposta, $nrVersao, $nrNovaVersao, $idDuplicaValores, $idAlteraIdTipoLP = null) {
        $linhasProduto = self::linhaProduto($nrProposta, $nrVersao);

        $manager = \Setforms2\Shared\Registry::getEntityManager();

        if (!empty($linhasProduto)) {
            foreach ($linhasProduto as $linha) {
                $propostaLinp = new Propostalinp();

                $propostaLinp->setNrproposfas($nrProposta);
                $propostaLinp->setNrorg($linha->getNrorg());                
                $propostaLinp->setNrverpropfas($nrNovaVersao);
                $propostaLinp->setCdlinpprop($linha->getCdlinpprop());                
                $propostaLinp->setNmlinpprop($linha->getNmlinpprop());                
                $propostaLinp->setQtlinpprop($linha->getQtlinpprop());
                $propostaLinp->setIdamblinpp($linha->getIdamblinpp());
                $propostaLinp->setDtinclusao(DateUtil::getDataAtual());
                $propostaLinp->setNrusualinpp($linha->getNrusualinpp());
                $propostaLinp->setNraceslinpp($linha->getNraceslinpp());
                $propostaLinp->setCdprecoprop($linha->getCdprecoprop());
                $propostaLinp->setDtcaluslinpp($linha->getDtcaluslinpp());
                $propostaLinp->setIdtppgtlinpp($linha->getIdtppgtlinpp());
                $propostaLinp->setIdlproclinpp($linha->getIdlproclinpp());
                $propostaLinp->setIdmproclinpp($linha->getIdmproclinpp());
                $propostaLinp->setQtminimafatprop($linha->getQtminimafatprop());
                $propostaLinp->setCdlinppropinfor($linha->getCdlinppropinfor());

                //Se for trocar o tipo de proposta a ser duplicada.
                if($idAlteraIdTipoLP) {
                    if($linha->getIdtiplinpp() == 'M') {//Prod. Manutenção
                        $propostaLinp->setIdtiplinpp('L');
                    } else if ($linha->getIdtiplinpp() == 'L') {//Prod. Locação
                        $propostaLinp->setIdtiplinpp('M');                        
                    } else if ($linha->getIdtiplinpp() == 'P') {//Pto. Manutençao
                        $propostaLinp->setIdtiplinpp('A');                        
                    } else if ($linha->getIdtiplinpp() == 'A') {//Pto. Locação
                        $propostaLinp->setIdtiplinpp('P');                        
                    }
                } else {
                    $propostaLinp->setIdtiplinpp($linha->getIdtiplinpp());  
                }

                if($idDuplicaValores) {
                    $propostaLinp->setVrlusoprop($linha->getVrlusoprop());
                    $propostaLinp->setVrmensprop($linha->getVrmensprop());
                    $propostaLinp->setVrdelusoprop($linha->getVrdelusoprop());
                    $propostaLinp->setVrdemensprop($linha->getVrdemensprop());
                    $propostaLinp->setVrlusopropptoa($linha->getVrlusopropptoa());
                    $propostaLinp->setVrmenspropptoa($linha->getVrmenspropptoa());
                    $propostaLinp->setVrlicencusoprop($linha->getVrlicencusoprop());
                    $propostaLinp->setVrmensalidaprop($linha->getVrmensalidaprop());
                    $propostaLinp->setVrdlusopropptoa($linha->getVrdlusopropptoa());
                    $propostaLinp->setVrdmenspropptoa($linha->getVrdmenspropptoa());
                    $propostaLinp->setVrdeslicusoprop($linha->getVrdeslicusoprop());
                    $propostaLinp->setVrdesmensalprop($linha->getVrdesmensalprop());
                    $propostaLinp->setVrlicencusotabe($linha->getVrlicencusotabe());
                    $propostaLinp->setVrmensalidatabe($linha->getVrmensalidatabe());
                }
                
                $propostaLinp->persist();

                $manager->flush();
                
                self::DuplicaSistemaLinhaProduto($nrProposta, $nrVersao, $nrNovaVersao, $linha->getCdlinpprop());
                
                if($idDuplicaValores) {
                    self::DuplicaParcelasNegociarLinhaProduto($nrProposta, $nrVersao, $nrNovaVersao, $linha->getCdlinpprop());
                }
            }
        }
    }

    public static function DuplicaSistemaLinhaProduto($nrProposta, $nrVersao, $nrNovaVersao, $cdLinpProp) {
        $sistemasLinhaProduto = Sistproplinp::getRepository()->sistemasLinhaProduto($nrProposta, $nrVersao, $cdLinpProp);

        $manager = \Setforms2\Shared\Registry::getEntityManager();

        foreach ($sistemasLinhaProduto as $sistema) {
            $sistPropLinp = new Sistproplinp();

            $sistPropLinp->setNrorg($sistema['NRORG']);
            $sistPropLinp->setNrproposfas($nrProposta);
            $sistPropLinp->setNrverpropfas($nrNovaVersao);
            $sistPropLinp->setCdsistema($sistema['CDSISTEMA']);
            $sistPropLinp->setCdlinpprop($sistema['CDLINPPROP']);
            $sistPropLinp->setDtinclusao(DateUtil::getDataAtual());
            $sistPropLinp->setDtinclusao(DateUtil::getDataDeString($sistema['DTINCLUSAO'], DateUtil::FORMATO_BRASILEIRO_DATAHORA));

            $sistPropLinp->persist();
        }

        $manager->flush();
    }
    
    public static function DuplicaParcelasNegociarLinhaProduto($nrProposta, $nrVersao, $nrNovaVersao, $cdLinpProp){
        $parcelasLinha = Venclusoprop::getRepository()->getParcelasNegociarLinhaProduto($nrProposta, $nrVersao, $cdLinpProp);
        $manager = \Setforms2\Shared\Registry::getEntityManager();

        if (!empty($parcelasLinha)) {
            $dtPrimVenc = $parcelasLinha[0]['DTVELUSOPEDP'];
            $dtPrimVenc = DateUtil::getDataDeString($dtPrimVenc, DateUtil::FORMATO_BRASILEIRO_DATAHORA, true);
            $dtAtual    = DateUtil::getDataAtual(true);

            if(DateUtil::getQtdeDiasEntre($dtAtual, $dtPrimVenc) < 7) {
                $dtPrimVenc = DateUtil::adicionaIntervalo($dtAtual, 10, DateUtil::DIA);
            }

            foreach ($parcelasLinha as $key => $linha) {
                $vencLinpProp = new Venclusoprop();
                $vencLinpProp->setNrorg($linha['NRORG']);
                $vencLinpProp->setNrproposfas($nrProposta);
                $vencLinpProp->setNrverpropfas($nrNovaVersao);
                $vencLinpProp->setCdlinpprop($linha['CDLINPPROP']);
                $vencLinpProp->setVrvelusopedp($linha['VRVELUSOPEDP']);
                $vencLinpProp->setDtinclusao(DateUtil::getDataAtual(true));

                if($key > 0) {
                    $dtPrimVenc = DateUtil::adicionaIntervalo($dtPrimVenc, 30, DateUtil::DIA);
                }

                $vencLinpProp->setDtvelusopedp($dtPrimVenc);

                $vencLinpProp->persist();
            }
            $manager->flush();
        }
    }

    public static function DuplicaServicos($nrProposta, $nrVersao, $nrNovaVersao) {
        $servicosProposta = self::propostaServ($nrProposta, $nrVersao);

        $manager = \Setforms2\Shared\Registry::getEntityManager();

        if (!empty($servicosProposta)) {
            foreach ($servicosProposta as $servico) {
                $propostaServ = new Propostaserv();

                $propostaServ->setNrproposfas($nrProposta);
                $propostaServ->setNrorg($servico->getNrorg());
                $propostaServ->setNrverpropfas($nrNovaVersao);
                $propostaServ->setDtinclusao(DateUtil::getDataAtual());
                $propostaServ->setNrservprop($servico->getNrservprop());
                $propostaServ->setCdtiposerv($servico->getCdtiposerv());
                $propostaServ->setVrhoraprop($servico->getVrhoraprop());
                $propostaServ->setQtservprop($servico->getQtservprop());
                $propostaServ->setDslocalprop($servico->getDslocalprop());
                $propostaServ->setVrdehoraprop($servico->getVrdehoraprop());
                $propostaServ->setIdtipocobrprop($servico->getIdtipocobrprop());

                $propostaServ->persist();               
            }
             $manager->flush();
        }
    }
    
    public static function DuplicaParcelasNegociarServicos($nrProposta, $nrVersao, $nrNovaVersao){
        $parcelasServicos = Vencservprop::getRepository()->getParcelasNegociarServicos($nrProposta, $nrVersao);
        $manager = \Setforms2\Shared\Registry::getEntityManager();

        if (!empty($parcelasServicos)) {
            $dtPrimVenc = $parcelasServicos[0]['DTVENCSERVPROP'];
            $dtPrimVenc = DateUtil::getDataDeString($dtPrimVenc, DateUtil::FORMATO_BRASILEIRO_DATAHORA, true);
            $dtAtual    = DateUtil::getDataAtual(true);

            if(DateUtil::getQtdeDiasEntre($dtAtual, $dtPrimVenc) < 7) {
                $dtPrimVenc = DateUtil::adicionaIntervalo($dtAtual, 10, DateUtil::DIA);
            }

            foreach ($parcelasServicos as $key => $servico) {
                $vencServProp = new Vencservprop();

                $vencServProp->setNrorg($servico['NRORG']);
                $vencServProp->setNrproposfas($nrProposta);
                $vencServProp->setNrverpropfas($nrNovaVersao);
                $vencServProp->setDtinclusao(DateUtil::getDataAtual(true));
                $vencServProp->setVrvencservprop($servico['VRVENCSERVPROP']);

                if($key > 0) {
                    $dtPrimVenc = DateUtil::adicionaIntervalo($dtPrimVenc, 30, DateUtil::DIA);
                }

                $vencServProp->setDtvencservprop($dtPrimVenc);

                $vencServProp->persist();                
            }
            $manager->flush();
        }
    }

    public static function deleteObsPropObjProp($nrProposta, $nrVersao, $idTipoProp)
    {
        Obspropfas::getRepository()->deleteObservacaoPropostaComercial($nrProposta, $nrVersao);

        if ($idTipoProp == 'S') {
            Objetpropfas::getRepository()->deleteObjetivoPropostaComercial($nrProposta, $nrVersao);
        }
    }

    public static function alteraTexto($dados)
    {
        //Garante que sempre a coluna do tipo Clob, campo com os textos considerados na nova intranet, tenha seu valor.
        self::setTxpropfasClob($dados['NRPROPOSFAS'], $dados['NRVERPROPFAS'], Ambiente::getNrorg(), 'EDICAO');

        $mnemonicos   = Propostafas::getRepository()->mnemonicoSoliRev($dados['NRPROPOSFAS'], $dados['NRVERPROPFAS']);
        $primeiraMens = Propostalinp::getRepository()->consultaPrimeiraMens($dados['NRPROPOSFAS'], $dados['NRVERPROPFAS']);

        //Início da Rotina para alterar o texto da Proposta.
        //Busca os mnemônicos no texto e troca pelo valor do campo cadastrado na tela.

        //Aproveitando função que traz todos os dados necessários. -> consultaDadosImprimirProposta().
        $consultaProp = Propostafas::getRepository()->consultaDadosImprimirProposta($dados['NRPROPOSFAS'], $dados['NRVERPROPFAS'], $dados['CDFILIAL']);

        //Deleta Mensagens do tipo IMPRESSAO
        self::deleteMensagemProposta($dados);

        //Busca Mensagens do tipo EDICAO
        $consMsgEdicao = Txtpropfas::getRepository()->consultaMensagemProp($dados['NRPROPOSFAS'], $dados['NRVERPROPFAS']);

        $manager = Registry::getEntityManager();
        $db = $manager->getConnection();
        $db->beginTransaction();

        try {
            foreach($consMsgEdicao as $txt) {
                //Mnemonicos a serem sobrepostos.
                $dadosAlterar = self::retornaArraysReplace('MNEMONICOS');

                //Dados para sobreposição.
                $dadosParaAlter = self::retornaArraysReplace('DADOS', $consultaProp, $mnemonicos, $primeiraMens);

                //Substitui os caracteres especiais por seu código Hexadecimal. Necessário passagem por Referência.
                foreach($dadosParaAlter as &$campo) {
                    $campo = self::replaceSpecialChars($campo, true);
                }

                $mensagem = $txt['TXPROPFAS_CLOB'];
                $msgFinal = str_replace($dadosAlterar, $dadosParaAlter, $mensagem);

                //Insere as novas mensagens do tipo IMPRESSAO e atualiza as do tipo EDICAO com os registros da proposta.
                self::insereMensagemProposta($dados, $txt, $msgFinal);

                //Passar o que está na coluna auxiliar, onde está o dado, para a coluna onde fica o Texto para Impressão, que é um Long Raw
                self::setTxpropfas($dados['NRPROPOSFAS'], $dados['NRVERPROPFAS'], Ambiente::getNrorg(), 'EDICAO');
                self::setTxpropfas($dados['NRPROPOSFAS'], $dados['NRVERPROPFAS'], Ambiente::getNrorg(), 'IMPRESSAO');
            }

            $db->commit();
        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }
    }
    
    public static function deleteMensagemProposta($dados)
    {
        $txtProp = Txtpropfas::findByNrproposfasAndNrverpropfasAndIdtipotxtpropfas($dados['NRPROPOSFAS'], $dados['NRVERPROPFAS'], 'IMPRESSAO');
        
        $manager = \Setforms2\Shared\Registry::getEntityManager();

        foreach ($txtProp as $txt) {
            if (is_object($txt)) {
                $txt->delete();
            }
        }
        
        $manager->flush();
    }

    public static function alteraMensagemEdicaoProposta($dados, $txt)
    {
        $txtProp = Txtpropfas::findOneByNrproposfasAndNrverpropfasAndIdtipotxtpropfasAndCdtxtpropfas($dados['NRPROPOSFAS'], $dados['NRVERPROPFAS'], 'EDICAO', $txt['CDTXTPROPFAS']);

        $manager = \Setforms2\Shared\Registry::getEntityManager();

        if (is_object($txtProp)) {
            $txtProp->setTxpropfasClob($txt['TXPROPFAS_CLOB']);
            $manager->flush();
        }
    }
    
    public static function insereMensagemProposta($dados, $txt, $msgFinal)
    {
        $manager = \Setforms2\Shared\Registry::getEntityManager();

        $txtPropFas = new Txtpropfas();

        $txtPropFas->setNrverpropfas($dados['NRVERPROPFAS']);
        $txtPropFas->setNrproposfas($dados['NRPROPOSFAS']);
        $txtPropFas->setCdtxtpropfas($txt['CDTXTPROPFAS']);
        $txtPropFas->setNrordemmsg($txt['NRORDEMMSG']);
        $txtPropFas->setIdtipotxtpropfas('IMPRESSAO');
        $txtPropFas->setDtinclusao(DateUtil::getDataAtual());
        $txtPropFas->setNrorg(Ambiente::getNrorg());
        $txtPropFas->setTxpropfasClob($msgFinal);
        $txtPropFas->setTxpropfas('Teste');
        $txtPropFas->persist();

        $manager->flush();
    }
    
    private static function retornaNomeMes($nrMes)
    {
        switch($nrMes) {
            case '01': $nmMes = 'Janeiro';   break;
            case '02': $nmMes = 'Fevereiro'; break;
            case '03': $nmMes = 'Março';     break;
            case '04': $nmMes = 'Abril';     break;
            case '05': $nmMes = 'Maio';      break;
            case '06': $nmMes = 'Junho';     break;
            case '07': $nmMes = 'Julho';     break;
            case '08': $nmMes = 'Agosto';    break;
            case '09': $nmMes = 'Setembro';  break;
            case '10': $nmMes = 'Outubro';   break;
            case '11': $nmMes = 'Novembro';  break;
            case '12': $nmMes = 'Dezembro';  break;
            default:   $nmMes = '';          break;
        }
        
        return $nmMes;
    }
    
    private static function retornaDataExtenso($data)
    {
        $data = $data->format(DateUtil::FORMATO_BRASILEIRO);

        $dia = substr($data, 0, 2);
        $mes = substr($data, 3, 2);
        $ano = substr($data, 6, 4);

        $dtExtenso = $dia . ' de ' . self::retornaNomeMes($mes) . ' de ' . $ano;

        return $dtExtenso;
    }
    
    private static function validaPosicaoArray($array, $posiçao)
    {
        if(isset($array[$posiçao])) {
            return $array[$posiçao];
        } else {
            return '';
        }
    }
    
    private static function retornaArraysReplace($tipeArray, $consultaProp = null, $mnemonicos = null, $primeiraMens = null)
    {
        if ($tipeArray == 'MNEMONICOS') {
            $dados = array('#PROPOSTA', '#VERSAO', '#DATAPROPOSTA', '#DATAVALIDADE', '#DATAACEITE', '#INTERLOCUTOR',
                           '#SOLICITANTE', '#RAZAOCLIENTE/PROSPECT', '#FANTASIACLIENTE/PROSPECT', '#INSCRICAOCLIENTE/PROSPECT',
                           '#ENDERECOCLIENTE/PROSPECT', '#PAISCLIENTE/PROSPECT', '#ESTADOCLIENTE/PROSPECT', '#MUNICIPIOCLIENTE/PROSPECT',
                           '#BAIRROCLIENTE/PROSPECT', '#CEPCLIENTE/PROSPECT', '#TELEFONECLIENTE/PROSPECT', '#RESIDENCIALCLIENTE/PROSPECT',
                           '#FAXCLIENTE/PROSPECT', '#EMAILCLIENTE/PROSPECT', '#CONTATOCLIENTE/PROSPECT', '#REFERENCIA', '#INDICE1',
                           '#INDICE2', '#PERCENTUAL1', '#PERCENTUAL2', '#REAJUSTECLIENTE/PROSPECT', '#MESREAJUSTE', '#RESPONSAVEL',
                           '#INCLUSAO', '#PRODUTOCOMERCIAL', '#TIPOPROPOSTAFAS', '#DATAATUALMESEXTENSO', '#DATAATUAL', '#CARGOOPERSOLI',
                           '#GERENTEOPERSOLI', '#CARGO_GERENTE_SOLICITANTE', '#CARGO_INTERLOCUTOR', '#NOME_FANTASIA_REVENDA',
                           '#RAZAO_SOCIAL_REVENDA', '#INTERLOCUTOR_REVENDA', '#CARGO_INTERLOCUTOR_REVENDA', '#MES_PRIMEIRO_FAT_MENSALIDADE',
                           '#MES_PRIMEIRO_PAG_MENSALIDADE', '#CPF_GERENTE_SOLICITANTE');
        } else {
            $mesReaj = self::retornaNomeMes($consultaProp[0]['NRMESREAJPROP']);
        
            if ($consultaProp[0]['IDTPREAJPROP'] == 'A') {
                $tipoReaj = 'Anual';
            } elseif ($consultaProp[0]['IDTPREAJPROP'] == 'M') {
                $tipoReaj = 'Bienal';
            }

            if ($consultaProp[0]['IDTIPOPROPFAS'] == 'M') {
                $tipoProp = 'LU/Manutenção';
            } else if ($consultaProp[0]['IDTIPOPROPFAS'] == 'L') {
                $tipoProp = 'Locação';
            } else if ($consultaProp[0]['IDTIPOPROPFAS'] == 'S') {
                $tipoProp = 'Serviços';
            }

            $dtAtual = DateUtil::getDataAtual();
            $dtAtualExtenso = self::retornaDataExtenso($dtAtual);
            
            //Posições q podem vir Nulas.
            $carInteRev = self::validaPosicaoArray($mnemonicos[0], 'NMCARGOINTECLIEREV');
            $nmInteRev  = self::validaPosicaoArray($mnemonicos[0], 'NMINTECLIEREV');
            $dsRefCont  = self::validaPosicaoArray($consultaProp[0], 'DSREFECONT');
            $dtAceite   = self::validaPosicaoArray($consultaProp[0], 'DTACEPROPFAS');
            
            if(!empty($primeiraMens)) {
                $dtMesPag   = self::validaPosicaoArray($primeiraMens[0], 'DTMESPAGMENSAL');
                $dtMesFat   = self::validaPosicaoArray($primeiraMens[0], 'DTMESFATMENSAL');
            } else {
                $dtMesPag = '';
                $dtMesFat = '';
            }
            
            $nrPropLtrim = ltrim($consultaProp[0]['NRPROPOSFAS'], '0');
            
            $dados = array($nrPropLtrim, $consultaProp[0]['NRVERPROPFAS'], $consultaProp[0]['DTPROPOSFAS'],
                           $consultaProp[0]['DTVALPROPFAS'], $dtAceite, $consultaProp[0]['NMINTECLIE'],
                           $consultaProp[0]['OPERSOLI'], $consultaProp[0]['RAZAOSOCIAL'], $consultaProp[0]['NOMEFANTASIA'],
                           $consultaProp[0]['NRINSJURCLIE'], $consultaProp[0]['ENDERECO'], $consultaProp[0]['PAIS'],
                           $consultaProp[0]['ESTADO'], $consultaProp[0]['MUNICIPIO'], $consultaProp[0]['BAIRRO'], $consultaProp[0]['CEP'],
                           $consultaProp[0]['TEL'], $consultaProp[0]['RESIDENCIAL'], $consultaProp[0]['NRFAXPROPFAS'],
                           $consultaProp[0]['DSEMAIPROFAS'], $dsRefCont, $consultaProp[0]['DSREFEPROPOSFAS'],
                           $consultaProp[0]['NMINDICEREAJ1'], $consultaProp[0]['NMINDICEREAJ2'],
                           number_format($consultaProp[0]['VRPEINCIND1'], 2, ',', '.') . '%',
                           number_format($consultaProp[0]['VRPEINCIND2'], 2, ',', '.') . '%',
                           $tipoReaj, $mesReaj, $consultaProp[0]['NMOPERRESP'], $consultaProp[0]['NMOPERADOR'],
                           $consultaProp[0]['NMPROCOMERCI'], $tipoProp, $dtAtualExtenso, $dtAtual, $mnemonicos[0]['NMCARGOOPERSOLI'],
                           $mnemonicos[0]['NMGEROPERSOLI'], $mnemonicos[0]['NMCARGEROPERSOLI'], $mnemonicos[0]['NMCARGOINTERLOC'],
                           $mnemonicos[0]['NMFANTCLIEREV'], $mnemonicos[0]['NMRAZSOCCLIEREV'], $nmInteRev, $carInteRev, $dtMesFat,
                           $dtMesPag, $mnemonicos[0]['NRCPFGERSOLI']);
        }
        
        return $dados;
    }
    
    public static function replaceSpecialChars($campo, $convertToHex = null)
    {
        //Array com possíveis caracteres especiais
        $specialChar = array('€', '‚', 'ƒ', '„', '…', '†', '‡', 'ˆ', '‰', 'Š', '‹', 'Ž', '‘', '’', '“', '”', '•', '–', '—', '˜', '™', 'š',
                             '›', 'ž', 'Ÿ', 'ç', '¡', '¢', '£', '¤', '¥', '¦', '§', '¨', '©', 'ª', '«', '¬', '®', '¯', '°', '±', '²', '³',
                             '´', 'μ', '¶', '·', '¸', '¹', 'º', '»', '¼', '½', '¾', '¿', 'À', 'Á', 'Â', 'Ã', 'Ä', 'Å', 'Æ', 'Ç', 'È', 'É',
                             'Ê', 'Ë', 'Ì', 'Í', 'Î', 'Ï', 'Ð', 'Ñ', 'Ò', 'Ó', 'Ô', 'Õ', 'Ö', '×', 'Ø', 'Ù', 'Ú', 'Û', 'Ü', 'Ý', 'Þ', 'ß',
                             'à', 'á', 'â', 'ã', 'ä', 'å', 'æ', 'ç', 'è', 'é', 'ê', 'ë', 'ì', 'í', 'î', 'ï', 'ð', 'ñ', 'ò', 'ó', 'ô', 'õ',
                             'ö', '÷', 'ø', 'ù', 'ú', 'û', 'ü', 'ý', 'þ', 'ÿ');
        
        //Array com seus códigos em Hexadecimal.
        //Obs.: o código real utiliza apenas uma barra. Ex.: \'80 (Primeiro item do array). 
        //                                                   As outras duas são para a barra do código e o apóstrofo funcionarem.
        $specialCharHex = array('\\\'80', '\\\'82', '\\\'83', '\\\'84', '\\\'85', '\\\'86', '\\\'87', '\\\'88', '\\\'89', '\\\'8a', '\\\'8b',
                                '\\\'8e', '\\\'91', '\\\'92', '\\\'93', '\\\'94', '\\\'95', '\\\'96', '\\\'97', '\\\'98', '\\\'99', '\\\'9a',
                                '\\\'9b', '\\\'9e', '\\\'9f', '\\\'e7', '\\\'a1', '\\\'a2', '\\\'a3', '\\\'a4', '\\\'a5', '\\\'a6', '\\\'a7',
                                '\\\'a8', '\\\'a9', '\\\'aa', '\\\'ab', '\\\'ac', '\\\'ae', '\\\'af', '\\\'b0', '\\\'b1', '\\\'b2', '\\\'b3',
                                '\\\'b4', '\\\'b5', '\\\'b6', '\\\'b7', '\\\'b8', '\\\'b9', '\\\'ba', '\\\'bb', '\\\'bc', '\\\'bd', '\\\'be',
                                '\\\'bf', '\\\'c0', '\\\'c1', '\\\'c2', '\\\'c3', '\\\'c4', '\\\'c5', '\\\'c6', '\\\'c7', '\\\'c8', '\\\'c9',
                                '\\\'ca', '\\\'cb', '\\\'cc', '\\\'cd', '\\\'ce', '\\\'cf', '\\\'d0', '\\\'d1', '\\\'d2', '\\\'d3', '\\\'d4',
                                '\\\'d5', '\\\'d6', '\\\'d7', '\\\'d8', '\\\'d9', '\\\'da', '\\\'db', '\\\'dc', '\\\'dd', '\\\'de', '\\\'df',
                                '\\\'e0', '\\\'e1', '\\\'e2', '\\\'e3', '\\\'e4', '\\\'e5', '\\\'e6', '\\\'e7', '\\\'e8', '\\\'e9', '\\\'ea',
                                '\\\'eb', '\\\'fc', '\\\'ed', '\\\'ee', '\\\'ef', '\\\'f0', '\\\'f1', '\\\'f2', '\\\'f3', '\\\'f4', '\\\'f5',
                                '\\\'f6', '\\\'f7', '\\\'f8', '\\\'f9', '\\\'fa', '\\\'fb', '\\\'fc', '\\\'fd', '\\\'fe', '\\\'ff');
        //Converter para Hexadecimal.
        if ($convertToHex) {
            $campo = str_replace($specialChar, $specialCharHex, $campo);
        } else { //Converter para Texto.
            $campo = str_replace($specialCharHex, $specialChar, $campo);
        }

        return $campo;
    }
    
    public static function retornaDataSet()
    {
        $mapper = \Setforms2\Runtime\Framework::getInstance()->getCurrentMapper();
        
        if($mapper instanceof \Setforms2\Runtime\Mapper\Crud) {
            $data = $mapper->getDatasource()->getData();
            $mapper->afterGetData($data);
            $data = $mapper->getDatasource()->getData();
            return $data;
        }
        return;        
    }
    
    public static function returnBoolTxtPropFas($nrProposta, $nrVersao, $idTipoTxt) 
    {
        $txtPropFas = Txtpropfas::findOneByNrproposfasAndNrverpropfasAndIdtipotxtpropfas($nrProposta, $nrVersao, $idTipoTxt);
        
        if(is_object($txtPropFas)) {
            return true;
        }
        return false;
    }
    
    public static function setValorDefaultCarenciaLP($nrProposta, $nrVersao, $dtProposta)
    {
        $manager = Registry::getEntityManager();
        $db = $manager->getConnection();
        $db->beginTransaction();

        try {
            $dtCarencia = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($dtProposta, DateUtil::FORMATO_BRASILEIRO_DATAHORA, true));

            $propLinp = Propostalinp::findByNrproposfasAndNrverpropfas($nrProposta, $nrVersao);

            foreach ($propLinp as $linhaProduto) {
                if(!$linhaProduto->getDtcaluslinpp()) {
                    self::setCarencia($linhaProduto, $dtCarencia);
                }
            }

            $db->commit();
        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }

    }
    
    public static function setCarencia(Propostalinp $linhaProduto, $data)
    {
        $linhaProduto->setDtcaluslinpp($data);
        $linhaProduto->persist();
        
        Registry::getEntityManager()->flush();
    }

    public static function getPermissaoOperador()
    {
        $cdoperador = Ambiente::getCdOperador();
        $nrorg      = Ambiente::getNrorg();

        $gruposDoOperador = Opergrupop::findByCdoperadorAndNrorg($cdoperador, $nrorg);

        $permissaoAlterar = false;
        foreach ($gruposDoOperador as $grupo) {
            $cdgrupoper = $grupo->getCdgrupoper();
            $autgrupop1 = Autgrupop::findByCdgrupoperAndIdautgrupopAndNrorg($cdgrupoper, 'PROP', $nrorg);
            $autgrupop2 = Autgrupop::findByCdgrupoperAndIdautgrupopAndNrorg($cdgrupoper, 'ACEP', $nrorg);

            //Permissão Máxima - utilizar todas as ferramentas.
            if(count($autgrupop2) > 0) {
                $permissaoAlterar = 'MAX';
                break;
            }

            //Permissão Mínima - limitação das ferramentas a utilizar, apenas visualizar todas as propostas.
            else if (count($autgrupop1) > 0) {
                $permissaoAlterar = 'MIN';
            }
        }

        return $permissaoAlterar;
    }

    public static function alteraTextoDuplicar($nrProposta, $nrVersao, $nrNovaVersao)
    {
        $txtProp = Txtpropfas::findByNrproposfasAndNrverpropfasAndIdtipotxtpropfas($nrProposta, $nrVersao, 'EDICAO');

        $entityManager = Registry::getEntityManager();

        foreach ($txtProp as $texto) {
            $textoNewProp = new \Geral\Entities\Txtpropfas();
            $textoNewProp->setDtinclusao(DateUtil::getDataAtual());
            $textoNewProp->setNrorg(Ambiente::getNrorg());
            $textoNewProp->setTxpropfasClob($texto->getTxpropfas());
            $textoNewProp->setCdtxtpropfas($texto->getCdtxtpropfas());
            $textoNewProp->setNrordemmsg($texto->getNrordemmsg());
            $textoNewProp->setNrproposfas($nrProposta);
            $textoNewProp->setNrverpropfas($nrNovaVersao);
            $textoNewProp->setIdtipotxtpropfas('EDICAO');
            $textoNewProp->setTxpropfas('Texto');

            $textoNewProp->persist();
        }

        $entityManager->flush();
    }

    public static function verificaProspectInativo($codigo)
    {
        $prospect = \Geral\Entities\Prospect::findOneByCdprospect($codigo);

        if(is_object($prospect)) {
            if($prospect->getIdativo() == 'S') {
                return true;
            }
        }

        return false;
    }

    public static function updateCdClienteProspect($inscricao, $idClienProsp, $nrProposta, $nrContato, $interlocutor)
    {
        $manager = Registry::getEntityManager();
        $db = $manager->getConnection();
        $db->beginTransaction();

        try {
            if($idClienProsp == 'C') {
                $cliente = Cliente::findOneByNrinsjurclieAndIdativo($inscricao, 'S');

                if(is_object($cliente)) {
                    $propostas = Propostafas::findByNrproposfas($nrProposta);

                    //Altera o cliente, o interlocutor e o contato de todas as versões da proposta.
                    foreach($propostas as $proposta) {
                        $proposta->setCdcliente($cliente->getCdcliente());
                        $proposta->setCdinteclie($interlocutor);
                        $proposta->setNrcontato($nrContato);

                        $proposta->persist();
                    }
                }
            } else {
                $prospect = Prospect::findOneByNrinsjurprosAndIdativo($inscricao, 'S');

                if(is_object($prospect)) {
                    $propostas = Propostafas::findByNrproposfas($nrProposta);

                    foreach($propostas as $proposta) {
                        $proposta->setCdprospect($prospect->getCdprospect());
                        $proposta->setCdintepros($interlocutor);
                        $proposta->setNrcontato($nrContato);

                        $proposta->persist();
                    }
                }
            }

            $manager->flush();
            $db->commit();
        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }
    }

    public static function reverterAgendamento($nrProposta, $nrVersao)
    {
        $manager = Registry::getEntityManager();
        $db = $manager->getConnection();
        $db->beginTransaction();

        try {
            $proposta = Propostafas::findOneByNrproposfasAndNrverpropfas($nrProposta, $nrVersao);

            if(is_object($proposta)) {
                $proposta->setIdliberaagendamento('N');
                $proposta->persist();
            }

            $manager->flush();
            $db->commit();
        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }
    }

    public static function reverterSituacao($nrProposta, $nrVersao)
    {
        $manager = Registry::getEntityManager();
        $db = $manager->getConnection();
        $db->beginTransaction();

        try {
            $proposta = Propostafas::findOneByNrproposfasAndNrverpropfas($nrProposta, $nrVersao);

            if(is_object($proposta)) {
                $proposta->setIdsolpropfas('S');
                $proposta->persist();
            }

            $manager->flush();
            $db->commit();
        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }
    }

    public static function alteraDadosFaturamento($params)
    {
        $manager = Registry::getEntityManager();
        $db = $manager->getConnection();
        $db->beginTransaction();

        try {
            $propostas  = Propostafas::findByNrproposfas($params['NRPROPOSFAS']);

            foreach ($propostas as $proposta) {
                $params['CDCLIENTE']    = $proposta->getCdcliente();
                $params['NRVERPROPFAS'] = $proposta->getNrverpropfas();

                self::alteraIndicesProposta($params);
                self::alteraMesReajContratoServ($params);
                self::alteraDatasValoresServ($params);
                self::alteraMesReajContratoLinp($params);
                self::alteraDatasValoresLinp($params);
            }

            self::alteraMesReajCliente($params);

            $manager->flush();
            $db->commit();
        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }
    }

    private static function alteraIndicesProposta($params)
    {
        $proposta = Propostafas::findOneByNrproposfasAndNrverpropfas($params['NRPROPOSFAS'], $params['NRVERPROPFAS']);

        if(is_object($proposta)) {
            $proposta->setCdindice1($params['CDINDICE1']);
            $proposta->setCdtpindice1($params['CDTPINDICE1']);
            $proposta->setIdpervalor1($params['IDPERVALOR1']);
            $proposta->setVrpeincind1($params['VRPEINCIND1']);
            $proposta->setCdindice2($params['CDINDICE2']);
            $proposta->setCdtpindice2($params['CDTPINDICE2']);
            $proposta->setIdpervalor2($params['IDPERVALOR2']);
            $proposta->setVrpeincind2($params['VRPEINCIND2']);
            $proposta->setNrmesreajprop($params['MESREAJUSTE']);
            $proposta->persist();
        }
    }

    private static function alteraMesReajContratoLinp($params)
    {
        $contratos = Contratolinp::findByCdcliente($params['CDCLIENTE']);

        foreach ($contratos as $contrato) {
            $contrato->setNrmesreajcont($params['MESREAJUSTE']);
            $contrato->persist();
        }
    }

    private static function alteraDatasValoresLinp($params)
    {
        $valores = Valorlinp::findByCdcliente($params['CDCLIENTE']);

        foreach ($valores as $valor) {
            $dtIniVig = date_parse($valor->getDtivigvrlinp());
            $dtFimVig = date_parse($valor->getDtfvigvrlinp());

            $reajusteIni = $params['MESREAJUSTE'];
            $reajusteFim = $params['MESREAJUSTE'] - 1;

            if($reajusteFim < 1) {
                $reajusteFim = 12;
                $dtFimVig['year'] = $dtFimVig['year'] - 1;
            }

            $newDtIniVig = str_pad($dtIniVig['day'], 2, '0', STR_PAD_LEFT).'/'.$reajusteIni.'/'.$dtIniVig['year'];
            $newDtFimVig = str_pad($dtFimVig['day'], 2, '0', STR_PAD_LEFT).'/'.$reajusteFim.'/'.$dtFimVig['year'];

            $newDtIniVig = DateUtil::getDataDeString($newDtIniVig, DateUtil::FORMATO_BRASILEIRO, true);
            $newDtFimVig = DateUtil::getDataDeString($newDtFimVig, DateUtil::FORMATO_BRASILEIRO, true);

            $valor->setDtivigvrlinp($newDtIniVig);
            $valor->setDtfvigvrlinp($newDtFimVig);

            $valor->persist();
        }
    }

    private static function alteraMesReajContratoServ($params)
    {
        $contratos = Contratoserv::findByNrproposfasAndNrverpropfas($params['NRPROPOSFAS'], $params['NRVERPROPFAS']);

        foreach ($contratos as $contrato) {
            $contrato->setNrmesreajserv($params['MESREAJUSTE']);
            $contrato->persist();

            self::alteraDataContratoServ($contrato, $params['MESREAJUSTE']);
        }
    }

    private static function alteraDataContratoServ(Contratoserv $contrato, $mesReajuste)
    {
        $dtIniVig = date_parse($contrato->getDtivigserv());
        $dtFimVig = date_parse($contrato->getDtfvigserv());

        $reajusteIni = $mesReajuste;
        $reajusteFim = $mesReajuste - 1;

        if($reajusteFim < 1) {
            $reajusteFim = 12;
            $dtFimVig['year'] = $dtFimVig['year'] - 1;
        }

        $newDtIniVig = str_pad($dtIniVig['day'], 2, '0', STR_PAD_LEFT).'/'.$reajusteIni.'/'.$dtIniVig['year'];
        $newDtFimVig = str_pad($dtFimVig['day'], 2, '0', STR_PAD_LEFT).'/'.$reajusteFim.'/'.$dtFimVig['year'];

        $newDtIniVig = DateUtil::getDataDeString($newDtIniVig, DateUtil::FORMATO_BRASILEIRO, true);
        $newDtFimVig = DateUtil::getDataDeString($newDtFimVig, DateUtil::FORMATO_BRASILEIRO, true);

        $contrato->setDtivigserv($newDtIniVig);
        $contrato->setDtfvigserv($newDtFimVig);

        $contrato->persist();
    }

    private static function alteraDatasValoresServ($params)
    {
        $valores = Valorserv::findByNrproposfasAndNrverpropfas($params['NRPROPOSFAS'], $params['NRVERPROPFAS']);

        foreach ($valores as $valor) {
            $dtIniVig = date_parse($valor->getDtivigvrserv());
            $dtFimVig = date_parse($valor->getDtfvigvrserv());

            $reajusteIni = $params['MESREAJUSTE'];
            $reajusteFim = $params['MESREAJUSTE'] - 1;

            if($reajusteFim < 1) {
                $reajusteFim = 12;
                $dtFimVig['year'] = $dtFimVig['year'] - 1;
            }

            $newDtIniVig = str_pad($dtIniVig['day'], 2, '0', STR_PAD_LEFT).'/'.$reajusteIni.'/'.$dtIniVig['year'];
            $newDtFimVig = str_pad($dtFimVig['day'], 2, '0', STR_PAD_LEFT).'/'.$reajusteFim.'/'.$dtFimVig['year'];

            $newDtIniVig = DateUtil::getDataDeString($newDtIniVig, DateUtil::FORMATO_BRASILEIRO, true);
            $newDtFimVig = DateUtil::getDataDeString($newDtFimVig, DateUtil::FORMATO_BRASILEIRO, true);

            $valor->setDtivigvrserv($newDtIniVig);
            $valor->setDtfvigvrserv($newDtFimVig);

            $valor->persist();
        }
    }

    public static function alteraMesReajCliente($params)
    {
        $cliente = Cliente::findOneByCdcliente($params['CDCLIENTE']);

        $cliente->setNrmesreajclie($params['MESREAJUSTE']);
        $cliente->persist();
    }

    public static function deleteLinhasProduto($nrProposta, $nrVersao, $nrOrg)
    {
        $manager = Registry::getEntityManager();
        $db = $manager->getConnection();
        $db->beginTransaction();

        try {
            $linhasProduto = Propostalinp::findByNrproposfasAndNrverpropfasAndNrorg($nrProposta, $nrVersao, $nrOrg);

            foreach ($linhasProduto as $linha) {
                //Deleta os Sistemas da Linha de Produto.
                LinhaProduto::deleteSistemasLinhaProduto($nrProposta, $nrVersao, $linha->getCdlinpprop(), $nrOrg);

                //Deleta as Parcelas as Linha de Produto.
                NegociarLinhaProduto::deleteParcelas($nrProposta, $nrVersao, $nrOrg, $linha->getCdlinpprop());

                //Por fim, deleta a Linha de Produto.
                $linha->delete();
            }

            $manager->flush();
            $db->commit();
        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }
    }

    public static function getNmProComerci($cdProcomerci)
    {
        $produto = Procomerci::findOneByCdprocomerci($cdProcomerci);

        if(is_object($produto)) {
            return $produto->getNmprocomerci();
        }

        return false;
    }
}